searchData={"items":[{"type":"module","title":"oidcc","doc":"OpenID Connect High Level Interface","ref":"oidcc.html"},{"type":"module","title":"Setup - oidcc","doc":"```erlang\n{ok, Pid} =\n  oidcc_provider_configuration_worker:start_link(#{\n    issuer => <<\"https://accounts.google.com\">>,\n    name => {local, google_config_provider}\n  }).\n```\n\n(or via a `m:supervisor`)\n\nSee `m:oidcc_provider_configuration_worker` for details","ref":"oidcc.html#module-setup"},{"type":"module","title":"Global Configuration - oidcc","doc":"* `max_clock_skew` (default `0`) - Maximum allowed clock skew for JWT\n  `exp` / `nbf` validation, in seconds","ref":"oidcc.html#module-global-configuration"},{"type":"function","title":"oidcc.client_credentials_token/4","doc":"Retrieve Client Credential Token.\n\nSee https://datatracker.ietf.org/doc/html/rfc6749#section-1.3.4.","ref":"oidcc.html#client_credentials_token/4"},{"type":"function","title":"Examples - oidcc.client_credentials_token/4","doc":"```erlang\n{ok, #oidcc_token{}} =\n  oidcc:client_credentials_token(\n    provider_name,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{scope => [<<\"scope\">>]}\n  ).\n```","ref":"oidcc.html#client_credentials_token/4-examples"},{"type":"function","title":"oidcc.create_redirect_url/4","doc":"Create Auth Redirect URL","ref":"oidcc.html#create_redirect_url/4"},{"type":"function","title":"Examples - oidcc.create_redirect_url/4","doc":"```erlang\n{ok, RedirectUri} =\n      oidcc:create_redirect_url(\n    provider_name,\n    <<\"client_id\">>,\n    <<\"client_secret\">>\n    #{redirect_uri: <<\"https://my.server/return\"}\n  ),\n\n%% RedirectUri = https://my.provider/auth?scope=openid&response_type=code&client_id=client_id&redirect_uri=https%3A%2F%2Fmy.server%2Freturn\n```","ref":"oidcc.html#create_redirect_url/4-examples"},{"type":"function","title":"oidcc.initiate_logout_url/4","doc":"Create Initiate URI for Relaying Party initiated Logout.\n\nSee https://openid.net/specs/openid-connect-rpinitiated-1_0.html#RPLogout.","ref":"oidcc.html#initiate_logout_url/4"},{"type":"function","title":"Examples - oidcc.initiate_logout_url/4","doc":"```erlang\n%% Get `Token` from `oidcc_token`\n\n{ok, RedirectUri} =\n  oidcc:initiate_logout_url(\n    Token,\n    provider_name,\n    <<\"client_id\">>,\n    #{post_logout_redirect_uri: <<\"https://my.server/return\"}}\n  ).\n\n%% RedirectUri = https://my.provider/logout?id_token_hint=IDToken&client_id=ClientId&post_logout_redirect_uri=https%3A%2F%2Fmy.server%2Freturn\n```","ref":"oidcc.html#initiate_logout_url/4-examples"},{"type":"function","title":"oidcc.introspect_token/5","doc":"Introspect the given access token.","ref":"oidcc.html#introspect_token/5"},{"type":"function","title":"Examples - oidcc.introspect_token/5","doc":"```erlang\n%% Get AccessToken\n\n{ok, #oidcc_token_introspection{active = True}} =\n  oidcc:introspect_token(\n    AccessToken,\n    provider_name,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{}\n  ).\n```","ref":"oidcc.html#introspect_token/5-examples"},{"type":"function","title":"oidcc.jwt_profile_token/6","doc":"Retrieve JSON Web Token (JWT) Profile Token.\n\nSee https://datatracker.ietf.org/doc/html/rfc7523#section-4.","ref":"oidcc.html#jwt_profile_token/6"},{"type":"function","title":"Examples - oidcc.jwt_profile_token/6","doc":"```erlang\n{ok, KeyJson} = file:read_file(\"jwt-profile.json\"),\nKeyMap = jose:decode(KeyJson),\nKey = jose_jwk:from_pem(maps:get(<<\"key\">>, KeyMap)),\n\n{ok, #oidcc_token{}} =\n  oidcc_token:jwt_profile(\n    <<\"subject\">>,\n    provider_name,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    Key,\n    #{\n     scope => [<<\"scope\">>],\n     kid => maps:get(<<\"keyId\">>, KeyMap)\n    }\n  ).\n```","ref":"oidcc.html#jwt_profile_token/6-examples"},{"type":"function","title":"oidcc.refresh_token/5","doc":"Refresh Token.","ref":"oidcc.html#refresh_token/5"},{"type":"function","title":"Examples - oidcc.refresh_token/5","doc":"```erlang\n%% Get Token and wait for its expiry\n\n{ok, #oidcc_token{}} =\n  oidcc:refresh_token(\n    Token,\n    provider_name,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{expected_subject => <<\"sub_from_initial_id_token\">>}\n  ).\n```","ref":"oidcc.html#refresh_token/5-examples"},{"type":"function","title":"oidcc.retrieve_token/5","doc":"Retrieve the token using the authcode received before and directly validate\nthe result.\n\nThe authcode was sent to the local endpoint by the OpenId Connect provider,\nusing redirects.","ref":"oidcc.html#retrieve_token/5"},{"type":"function","title":"Examples - oidcc.retrieve_token/5","doc":"```erlang\n%% Get AuthCode from Redirect\n\n{ok, #oidcc_token{}} =\n  oidcc:retrieve_token(\n    AuthCode,\n    provider_name,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{redirect_uri => <<\"https://example.com/callback\">>}\n  ).\n```","ref":"oidcc.html#retrieve_token/5-examples"},{"type":"function","title":"oidcc.retrieve_userinfo/5","doc":"Load userinfo for the given token.","ref":"oidcc.html#retrieve_userinfo/5"},{"type":"function","title":"Examples - oidcc.retrieve_userinfo/5","doc":"```erlang\n%% Get Token\n\n{ok, #{<<\"sub\">> => Sub}} =\n  oidcc:retrieve_userinfo(\n    Token,\n    provider_name,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{}\n  ).\n```","ref":"oidcc.html#retrieve_userinfo/5-examples"},{"type":"module","title":"oidcc_auth_util","doc":"Authentication Utilities","ref":"oidcc_auth_util.html"},{"type":"type","title":"oidcc_auth_util.auth_method/0","doc":"","ref":"oidcc_auth_util.html#t:auth_method/0"},{"type":"type","title":"oidcc_auth_util.error/0","doc":"","ref":"oidcc_auth_util.html#t:error/0"},{"type":"module","title":"oidcc_authorization","doc":"Functions to start an OpenID Connect Authorization","ref":"oidcc_authorization.html"},{"type":"function","title":"oidcc_authorization.create_redirect_url/2","doc":"Create Auth Redirect URL.\n\nFor a high level interface using `m:oidcc_provider_configuration_worker`\nsee `oidcc:create_redirect_url/4`.","ref":"oidcc_authorization.html#create_redirect_url/2"},{"type":"function","title":"Examples - oidcc_authorization.create_redirect_url/2","doc":"```erlang\n{ok, ClientContext} =\n    oidcc_client_context:from_configuration_worker(provider_name,\n                                                   <<\"client_id\">>,\n                                                   <<\"client_secret\">>),\n\n{ok, RedirectUri} =\n    oidcc_authorization:create_redirect_url(ClientContext,\n                                            #{redirect_uri: <<\"https://my.server/return\">}),\n\n%% RedirectUri = https://my.provider/auth?scope=openid&response_type=code&client_id=client_id&redirect_uri=https%3A%2F%2Fmy.server%2Freturn\n```","ref":"oidcc_authorization.html#create_redirect_url/2-examples"},{"type":"type","title":"oidcc_authorization.error/0","doc":"","ref":"oidcc_authorization.html#t:error/0"},{"type":"type","title":"oidcc_authorization.opts/0","doc":"Configure authorization redirect URL.\n\nSee https://openid.net/specs/openid-connect-core-1_0.html#AuthRequest.","ref":"oidcc_authorization.html#t:opts/0"},{"type":"type","title":"Parameters - oidcc_authorization.opts/0","doc":"* `scopes` - list of scopes to request (defaults to `[<<\"openid\">>]`)\n* `state` - state to pass to the provider\n* `nonce` - nonce to pass to the provider\n* `purpose` - purpose of the authorization request, see [https://cdn.connectid.com.au/specifications/oauth2-purpose-01.html]\n* `require_purpose` - whether to require a `purpose` value\n* `pkce_verifier` - PKCE verifier (random string), see [https://datatracker.ietf.org/doc/html/rfc7636#section-4.1]\n* `require_pkce` - whether to require PKCE when getting the token\n* `redirect_uri` - redirect target after authorization is completed\n* `url_extension` - add custom query parameters to the authorization URL\n* `response_mode` - response mode to use (defaults to `<<\"query\">>`)","ref":"oidcc_authorization.html#t:opts/0-parameters"},{"type":"module","title":"oidcc_backoff","doc":"Backoff Handling\n\nBased on [`db_connection`](https://github.com/elixir-ecto/db_connection/blob/8ef1f2ea54922873590b8939f2dad6b031c5b49c/lib/db_connection/backoff.ex#L24)","ref":"oidcc_backoff.html"},{"type":"type","title":"oidcc_backoff.max/0","doc":"","ref":"oidcc_backoff.html#t:max/0"},{"type":"type","title":"oidcc_backoff.min/0","doc":"","ref":"oidcc_backoff.html#t:min/0"},{"type":"opaque","title":"oidcc_backoff.state/0","doc":"","ref":"oidcc_backoff.html#t:state/0"},{"type":"type","title":"oidcc_backoff.type/0","doc":"","ref":"oidcc_backoff.html#t:type/0"},{"type":"module","title":"oidcc_client_context","doc":"Client Configuration for authorization, token exchange, and userinfo.\n\nFor most projects, it makes sense to use `m:oidcc_provider_configuration_worker` and the high-level\ninterface of `oidcc`. In that case, direct usage of this module is not needed.\n\nTo use the record, import the definition:\n\n```erlang\n-include_lib([\"oidcc/include/oidcc_client_context.hrl\"]).\n```","ref":"oidcc_client_context.html"},{"type":"function","title":"oidcc_client_context.apply_profiles/2","doc":"Apply OpenID Connect / OAuth2 Profiles to the context.\n\nCurrently, the only supported profiles are:\n- `fapi2_security_profile` - https://openid.bitbucket.io/fapi/fapi-2_0-security-profile.html\n- `fapi2_message_signing` - https://openid.bitbucket.io/fapi/fapi-2_0-message-signing.html\n\nIt returns an updated `t:t/0` record and a map of options to\nbe merged into the `m:oidcc_authorization` and `m:oidcc_token` functions.","ref":"oidcc_client_context.html#apply_profiles/2"},{"type":"function","title":"Examples - oidcc_client_context.apply_profiles/2","doc":"```erlang\nClientContext = #oidcc_client_context{} = oidcc_client_context:from_...(...),\n\n{#oidcc_client_context{} = ClientContext1, Opts} = oidcc_client_context:apply_profiles(\n  ClientContext,\n  #{\n    profiles => [fapi2_message_signing]\n  }),\n\n{ok, Uri} = oidcc_authorization:create_redirect_uri(\n  ClientContext1,\n  maps:merge(Opts, #{...})\n).\n```","ref":"oidcc_client_context.html#apply_profiles/2-examples"},{"type":"function","title":"oidcc_client_context.from_configuration_worker/3","doc":"Create Client Context from a `m:oidcc_provider_configuration_worker`.\n\nSee `from_configuration_worker/4`.","ref":"oidcc_client_context.html#from_configuration_worker/3"},{"type":"function","title":"oidcc_client_context.from_configuration_worker/4","doc":"Create Client Context from a `m:oidcc_provider_configuration_worker`.","ref":"oidcc_client_context.html#from_configuration_worker/4"},{"type":"function","title":"Examples - oidcc_client_context.from_configuration_worker/4","doc":"```erlang\n{ok, Pid} =\n  oidcc_provider_configuration_worker:start_link(#{\n    issuer => <<\"https://login.salesforce.com\">>\n  }),\n\n{ok, #oidcc_client_context{}} =\n  oidcc_client_context:from_configuration_worker(Pid,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>).\n```\n\n```erlang\n{ok, Pid} =\n  oidcc_provider_configuration_worker:start_link(#{\n    issuer => <<\"https://login.salesforce.com\">>,\n    name => {local, salesforce_provider}\n  }),\n\n{ok, #oidcc_client_context{}} =\n  oidcc_client_context:from_configuration_worker(\n    salesforce_provider,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{client_jwks => jose_jwk:generate_key(16)}\n  ).\n```","ref":"oidcc_client_context.html#from_configuration_worker/4-examples"},{"type":"function","title":"oidcc_client_context.from_manual/4","doc":"Create Client Context manually.\n\nSee `from_manual/5`.","ref":"oidcc_client_context.html#from_manual/4"},{"type":"function","title":"oidcc_client_context.from_manual/5","doc":"Create Client Context manually.","ref":"oidcc_client_context.html#from_manual/5"},{"type":"function","title":"Examples - oidcc_client_context.from_manual/5","doc":"```erlang\n{ok, Configuration} =\n  oidcc_provider_configuration:load_configuration(<<\"https://login.salesforce.com\">>, []),\n\n#oidcc_provider_configuration{jwks_uri = JwksUri} = Configuration,\n\n{ok, Jwks} = oidcc_provider_configuration:load_jwks(JwksUri, []).\n\n#oidcc_client_context{} =\n  oidcc_client_context:from_manual(\n    Metadata,\n    Jwks,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{client_jwks => jose_jwk:generate_key(16)}\n  ).\n```","ref":"oidcc_client_context.html#from_manual/5-examples"},{"type":"type","title":"oidcc_client_context.authenticated_opts/0","doc":"","ref":"oidcc_client_context.html#t:authenticated_opts/0"},{"type":"type","title":"oidcc_client_context.authenticated_t/0","doc":"","ref":"oidcc_client_context.html#t:authenticated_t/0"},{"type":"type","title":"oidcc_client_context.error/0","doc":"","ref":"oidcc_client_context.html#t:error/0"},{"type":"type","title":"oidcc_client_context.opts/0","doc":"","ref":"oidcc_client_context.html#t:opts/0"},{"type":"type","title":"oidcc_client_context.t/0","doc":"","ref":"oidcc_client_context.html#t:t/0"},{"type":"type","title":"oidcc_client_context.unauthenticated_opts/0","doc":"","ref":"oidcc_client_context.html#t:unauthenticated_opts/0"},{"type":"type","title":"oidcc_client_context.unauthenticated_t/0","doc":"","ref":"oidcc_client_context.html#t:unauthenticated_t/0"},{"type":"module","title":"oidcc_client_registration","doc":"Dynamic Client Registration Utilities.\n\nSee https://openid.net/specs/openid-connect-registration-1_0.html#ClientMetadata.","ref":"oidcc_client_registration.html"},{"type":"module","title":"Records - oidcc_client_registration","doc":"To use the record, import the definition:\n\n```erlang\n-include_lib([\"oidcc/include/oidcc_client_registration.hrl\"]).\n```","ref":"oidcc_client_registration.html#module-records"},{"type":"module","title":"Telemetry - oidcc_client_registration","doc":"See [`Oidcc.ClientRegistration`](`m:'Elixir.Oidcc.ClientRegistration'`).","ref":"oidcc_client_registration.html#module-telemetry"},{"type":"function","title":"oidcc_client_registration.register/3","doc":"Register Client.","ref":"oidcc_client_registration.html#register/3"},{"type":"function","title":"Examples - oidcc_client_registration.register/3","doc":"```erlang\n{ok, ProviderConfiguration} =\n  oidcc_provider_configuration:load_configuration(\"https://your.issuer\"),\n\n{ok, #oidcc_client_registration_response{\n  client_id = ClientId,\n  client_secret = ClientSecret\n}} =\n  oidcc_client_registration:register(\n    ProviderConfiguration,\n    #oidcc_client_registration{\n      redirect_uris = [\"https://your.application.com/oidcc/callback\"]\n    },\n    #{initial_access_token => <<\"optional token you got from the provider\">>}\n  ).\n```","ref":"oidcc_client_registration.html#register/3-examples"},{"type":"type","title":"oidcc_client_registration.error/0","doc":"","ref":"oidcc_client_registration.html#t:error/0"},{"type":"type","title":"oidcc_client_registration.opts/0","doc":"Configure configuration loading / parsing.","ref":"oidcc_client_registration.html#t:opts/0"},{"type":"type","title":"Parameters - oidcc_client_registration.opts/0","doc":"* `initial_access_token` - Access Token for registration\n* `request_opts` - config for HTTP request","ref":"oidcc_client_registration.html#t:opts/0-parameters"},{"type":"type","title":"oidcc_client_registration.response/0","doc":"Record containing Client Registration Response.\n\nSee https://openid.net/specs/openid-connect-registration-1_0.html#RegistrationResponse.\n\nAll unrecognized fields are stored in `extra_fields`.","ref":"oidcc_client_registration.html#t:response/0"},{"type":"type","title":"oidcc_client_registration.t/0","doc":"Record containing Client Registration Metadata.\n\nSee https://openid.net/specs/openid-connect-registration-1_0.html#ClientMetadata and\nhttps://openid.net/specs/openid-connect-rpinitiated-1_0.html#ClientMetadata.\n\nAll unrecognized fields are stored in `extra_fields`.","ref":"oidcc_client_registration.html#t:t/0"},{"type":"module","title":"oidcc_decode_util","doc":"Response Decoding Utils","ref":"oidcc_decode_util.html"},{"type":"type","title":"oidcc_decode_util.error/0","doc":"","ref":"oidcc_decode_util.html#t:error/0"},{"type":"module","title":"oidcc_http_util","doc":"HTTP Client Utilities","ref":"oidcc_http_util.html"},{"type":"function","title":"oidcc_http_util.headers_to_cache_deadline/2","doc":"","ref":"oidcc_http_util.html#headers_to_cache_deadline/2"},{"type":"type","title":"oidcc_http_util.error/0","doc":"","ref":"oidcc_http_util.html#t:error/0"},{"type":"type","title":"oidcc_http_util.http_header/0","doc":"See `httpc:request/5`.","ref":"oidcc_http_util.html#t:http_header/0"},{"type":"type","title":"oidcc_http_util.httpc_error/0","doc":"See `httpc:request/5` for additional errors.","ref":"oidcc_http_util.html#t:httpc_error/0"},{"type":"type","title":"oidcc_http_util.query_params/0","doc":"See `uri_string:compose_query/1`.","ref":"oidcc_http_util.html#t:query_params/0"},{"type":"type","title":"oidcc_http_util.request_opts/0","doc":"See `httpc:request/5`.","ref":"oidcc_http_util.html#t:request_opts/0"},{"type":"type","title":"Parameters - oidcc_http_util.request_opts/0","doc":"* `timeout` - timeout for request\n* `ssl` - TLS config","ref":"oidcc_http_util.html#t:request_opts/0-parameters"},{"type":"type","title":"oidcc_http_util.telemetry_opts/0","doc":"","ref":"oidcc_http_util.html#t:telemetry_opts/0"},{"type":"module","title":"oidcc_jwt_util","doc":"JWT Utilities","ref":"oidcc_jwt_util.html"},{"type":"type","title":"oidcc_jwt_util.claims/0","doc":"","ref":"oidcc_jwt_util.html#t:claims/0"},{"type":"type","title":"oidcc_jwt_util.error/0","doc":"","ref":"oidcc_jwt_util.html#t:error/0"},{"type":"type","title":"oidcc_jwt_util.refresh_jwks_for_unknown_kid_fun/0","doc":"","ref":"oidcc_jwt_util.html#t:refresh_jwks_for_unknown_kid_fun/0"},{"type":"module","title":"oidcc_logout","doc":"Logout from the OpenID Provider.","ref":"oidcc_logout.html"},{"type":"function","title":"oidcc_logout.initiate_url/3","doc":"Initiate URI for Relaying Party initiated Logout.\n\nSee https://openid.net/specs/openid-connect-rpinitiated-1_0.html#RPLogout.\n\nFor a high level interface using `m:oidcc_provider_configuration_worker`\nsee `oidcc:initiate_logout_url/4`.","ref":"oidcc_logout.html#initiate_url/3"},{"type":"function","title":"Examples - oidcc_logout.initiate_url/3","doc":"```erlang\n{ok, ClientContext} = oidcc_client_context:from_configuration_worker(\n  provider_name,\n  <<\"client_id\">>,\n  unauthenticated\n),\n\n%% Get `Token` from `oidcc_token`\n\n{ok, RedirectUri} =\n  oidcc_logout:initiate_url(\n    Token,\n    ClientContext,\n    #{post_logout_redirect_uri: <<\"https://my.server/return\">}\n),\n\n%% RedirectUri = https://my.provider/logout?id_token_hint=IDToken&client_id=ClientId&post_logout_redirect_uri=https%3A%2F%2Fmy.server%2Freturn\n```","ref":"oidcc_logout.html#initiate_url/3-examples"},{"type":"type","title":"oidcc_logout.error/0","doc":"","ref":"oidcc_logout.html#t:error/0"},{"type":"type","title":"oidcc_logout.initiate_url_opts/0","doc":"Configure Relaying Party initiated Logout URI.\n\nSee https://openid.net/specs/openid-connect-rpinitiated-1_0.html#RPLogout.","ref":"oidcc_logout.html#t:initiate_url_opts/0"},{"type":"type","title":"Parameters - oidcc_logout.initiate_url_opts/0","doc":"* `logout_hint` - logout_hint to pass to the provider\n* `post_logout_redirect_uri` - Post Logout Redirect URI to pass to the provider\n* `state` - state to pass to the provider\n* `ui_locales` - UI locales to pass to the provider\n* `extra_query_params` - extra query params to add to the URI","ref":"oidcc_logout.html#t:initiate_url_opts/0-parameters"},{"type":"module","title":"oidcc_profile","doc":"OpenID Profile Utilities","ref":"oidcc_profile.html"},{"type":"type","title":"oidcc_profile.error/0","doc":"","ref":"oidcc_profile.html#t:error/0"},{"type":"type","title":"oidcc_profile.opts/0","doc":"","ref":"oidcc_profile.html#t:opts/0"},{"type":"type","title":"oidcc_profile.opts_no_profiles/0","doc":"","ref":"oidcc_profile.html#t:opts_no_profiles/0"},{"type":"type","title":"oidcc_profile.profile/0","doc":"","ref":"oidcc_profile.html#t:profile/0"},{"type":"module","title":"oidcc_provider_configuration","doc":"Tooling to load and parse Openid Configuration.","ref":"oidcc_provider_configuration.html"},{"type":"module","title":"Records - oidcc_provider_configuration","doc":"To use the record, import the definition:\n\n```erlang\n-include_lib([\"oidcc/include/oidcc_provider_configuration.hrl\"]).\n```","ref":"oidcc_provider_configuration.html#module-records"},{"type":"module","title":"Telemetry - oidcc_provider_configuration","doc":"See [`Oidcc.ProviderConfiguration`](`m:'Elixir.Oidcc.ProviderConfiguration'`).","ref":"oidcc_provider_configuration.html#module-telemetry"},{"type":"function","title":"oidcc_provider_configuration.decode_configuration/1","doc":"See `decode_configuration/2`.","ref":"oidcc_provider_configuration.html#decode_configuration/1"},{"type":"function","title":"oidcc_provider_configuration.decode_configuration/2","doc":"Decode JSON into a `t:oidcc_provider_configuration:t/0` record.","ref":"oidcc_provider_configuration.html#decode_configuration/2"},{"type":"function","title":"Examples - oidcc_provider_configuration.decode_configuration/2","doc":"```erlang\n{ok, {{\"HTTP/1.1\",200,\"OK\"}, _Headers, Body}} =\n  httpc:request(\"https://accounts.google.com/.well-known/openid-configuration\"),\n\n{ok, DecodedJson} = your_json_lib:decode(Body),\n\n{ok, #oidcc_provider_configuration{}} =\n  oidcc_provider_configuration:decode_configuration(DecodedJson).\n```","ref":"oidcc_provider_configuration.html#decode_configuration/2-examples"},{"type":"function","title":"oidcc_provider_configuration.load_configuration/1","doc":"See `load_configuration/2`.","ref":"oidcc_provider_configuration.html#load_configuration/1"},{"type":"function","title":"oidcc_provider_configuration.load_configuration/2","doc":"Load OpenID Configuration into a `t:oidcc_provider_configuration:t/0` record.","ref":"oidcc_provider_configuration.html#load_configuration/2"},{"type":"function","title":"Examples - oidcc_provider_configuration.load_configuration/2","doc":"```erlang\n{ok, #oidcc_provider_configuration{}} =\n  oidcc_provider_configuration:load_configuration(\"https://accounts.google.com\").\n```","ref":"oidcc_provider_configuration.html#load_configuration/2-examples"},{"type":"function","title":"oidcc_provider_configuration.load_jwks/2","doc":"Load JWKs into a `t:jose_jwk:key/0` record.","ref":"oidcc_provider_configuration.html#load_jwks/2"},{"type":"function","title":"Examples - oidcc_provider_configuration.load_jwks/2","doc":"```erlang\n{ok, #jose_jwk{}} =\n  oidcc_provider_configuration:load_jwks(\"https://www.googleapis.com/oauth2/v3/certs\").\n```","ref":"oidcc_provider_configuration.html#load_jwks/2-examples"},{"type":"type","title":"oidcc_provider_configuration.error/0","doc":"","ref":"oidcc_provider_configuration.html#t:error/0"},{"type":"type","title":"oidcc_provider_configuration.opts/0","doc":"Configure configuration loading / parsing.","ref":"oidcc_provider_configuration.html#t:opts/0"},{"type":"type","title":"Parameters - oidcc_provider_configuration.opts/0","doc":"* `fallback_expiry` - How long to keep configuration cached if the server doesn't specify expiry.\n* `request_opts` - config for HTTP request.","ref":"oidcc_provider_configuration.html#t:opts/0-parameters"},{"type":"type","title":"oidcc_provider_configuration.quirks/0","doc":"Allow Specification Non-compliance.","ref":"oidcc_provider_configuration.html#t:quirks/0"},{"type":"type","title":"Exceptions - oidcc_provider_configuration.quirks/0","doc":"* `allow_unsafe_http` - Allow unsafe HTTP. Use this for development\n  providers and **never in production**.\n* `document_overrides` - a map to merge with the real OIDD document,\n  in case the OP left out some values.","ref":"oidcc_provider_configuration.html#t:quirks/0-exceptions"},{"type":"type","title":"oidcc_provider_configuration.t/0","doc":"Record containing OpenID and OAuth 2.0 Configuration.\n\nSee:\n* https://openid.net/specs/openid-connect-discovery-1_0.html#ProviderMetadata\n* https://datatracker.ietf.org/doc/html/draft-jones-oauth-discovery-01#section-4.1\n* https://openid.net/specs/openid-connect-rpinitiated-1_0.html#OPMetadata\n\nAll unrecognized fields are stored in `extra_fields`.","ref":"oidcc_provider_configuration.html#t:t/0"},{"type":"module","title":"oidcc_provider_configuration_worker","doc":"OIDC Config Provider Worker\n\nLoads and continuously refreshes the OIDC configuration and JWKs.\n\nThe worker supports reading values concurrently via an ETS table. To use\nthis performance improvement, the worker has to be registered with a\n`{local, Name}`. No name / `{global, Name}` and `{via, RegModule, ViaName}`\nare not supported.","ref":"oidcc_provider_configuration_worker.html"},{"type":"function","title":"oidcc_provider_configuration_worker.get_jwks/1","doc":"Get Parsed Jwks.","ref":"oidcc_provider_configuration_worker.html#get_jwks/1"},{"type":"function","title":"oidcc_provider_configuration_worker.get_provider_configuration/1","doc":"Get Configuration.","ref":"oidcc_provider_configuration_worker.html#get_provider_configuration/1"},{"type":"function","title":"oidcc_provider_configuration_worker.refresh_configuration/1","doc":"Refresh Configuration.","ref":"oidcc_provider_configuration_worker.html#refresh_configuration/1"},{"type":"function","title":"Examples - oidcc_provider_configuration_worker.refresh_configuration/1","doc":"```erlang\n{ok, Pid} =\n  oidcc_provider_configuration_worker:start_link(#{\n    issuer => <<\"https://accounts.google.com\">>\n  }).\n\n%% Later\n\noidcc_provider_configuration_worker:refresh_configuration(Pid).\n```","ref":"oidcc_provider_configuration_worker.html#refresh_configuration/1-examples"},{"type":"function","title":"oidcc_provider_configuration_worker.refresh_jwks/1","doc":"Refresh JWKs.","ref":"oidcc_provider_configuration_worker.html#refresh_jwks/1"},{"type":"function","title":"Examples - oidcc_provider_configuration_worker.refresh_jwks/1","doc":"```erlang\n{ok, Pid} =\n  oidcc_provider_configuration_worker:start_link(#{\n    issuer => <<\"https://accounts.google.com\">>\n  }).\n\n%% Later\n\noidcc_provider_configuration_worker:refresh_jwks(Pid).\n```","ref":"oidcc_provider_configuration_worker.html#refresh_jwks/1-examples"},{"type":"function","title":"oidcc_provider_configuration_worker.refresh_jwks_for_unknown_kid/2","doc":"Refresh JWKs if the provided `Kid` is not matching any currently loaded keys.","ref":"oidcc_provider_configuration_worker.html#refresh_jwks_for_unknown_kid/2"},{"type":"function","title":"Examples - oidcc_provider_configuration_worker.refresh_jwks_for_unknown_kid/2","doc":"```erlang\n{ok, Pid} =\n  oidcc_provider_configuration_worker:start_link(#{\n    issuer => <<\"https://accounts.google.com\">>\n  }).\n\noidcc_provider_configuration_worker:refresh_jwks_for_unknown_kid(Pid, <<\"kid\">>).\n```","ref":"oidcc_provider_configuration_worker.html#refresh_jwks_for_unknown_kid/2-examples"},{"type":"function","title":"oidcc_provider_configuration_worker.start_link/1","doc":"Start Configuration Provider.","ref":"oidcc_provider_configuration_worker.html#start_link/1"},{"type":"function","title":"Examples - oidcc_provider_configuration_worker.start_link/1","doc":"```erlang\n{ok, Pid} =\n  oidcc_provider_configuration_worker:start_link(#{\n    issuer => <<\"https://accounts.google.com\">>,\n    name => {local, google_config_provider}\n  }).\n```\n\n```erlang\n%% ...\n-behaviour(supervisor).\n\n%% ...\n\ninit(_opts) ->\n  SupFlags = #{strategy => one_for_one, intensity => 1, period => 5},\n  ChildSpecs = [#{id => google_config_provider,\n    start => {oidcc_provider_configuration_worker,\n              start_link,\n              [\n                #{issuer => <<\"https://accounts.google.com\">>}\n              ]},\n    restart => permanent,\n    type => worker,\n    modules => [oidcc_provider_configuration_worker]}],\n  {ok, {SupFlags, ChildSpecs}}.\n```","ref":"oidcc_provider_configuration_worker.html#start_link/1-examples"},{"type":"type","title":"oidcc_provider_configuration_worker.opts/0","doc":"Configuration Options\n\n* `name` - The gen_server name of the provider.\n* `issuer` - The issuer URI.\n* `provider_configuration_opts` - Options for the provider configuration\n  fetching.\n* `backoff_min` - The minimum backoff interval in ms (default: `1_000`).\n* `backoff_max` - The maximum backoff interval in ms (default: `30_000`).\n* `backoff_type` - The backoff strategy, `stop` for no backoff and to stop,\n  `exponential` for exponential, `random` for random, and `random_exponential`\n  for random exponential (default: `stop`).","ref":"oidcc_provider_configuration_worker.html#t:opts/0"},{"type":"module","title":"oidcc_scope","doc":"OpenID Scope Utilities","ref":"oidcc_scope.html"},{"type":"function","title":"oidcc_scope.parse/1","doc":"Parse `t:t/0` into `t:scopes/0`.","ref":"oidcc_scope.html#parse/1"},{"type":"function","title":"Examples - oidcc_scope.parse/1","doc":"```erlang\n[<<\"openid\">>, <<\"profile\">>] = oidcc_scope:parse(<<\"openid profile\">>).\n```","ref":"oidcc_scope.html#parse/1-examples"},{"type":"function","title":"oidcc_scope.scopes_to_bin/1","doc":"Compose `t:scopes/0` into `t:t/0`.","ref":"oidcc_scope.html#scopes_to_bin/1"},{"type":"function","title":"Examples - oidcc_scope.scopes_to_bin/1","doc":"```erlang\n<<\"openid profile email\">> = oidcc_scope:scopes_to_bin(\n  [<<\"openid\">>, profile, \"email\"]).\n```","ref":"oidcc_scope.html#scopes_to_bin/1-examples"},{"type":"type","title":"oidcc_scope.scopes/0","doc":"","ref":"oidcc_scope.html#t:scopes/0"},{"type":"type","title":"oidcc_scope.t/0","doc":"","ref":"oidcc_scope.html#t:t/0"},{"type":"module","title":"oidcc_token","doc":"Facilitate OpenID Code/Token Exchanges.","ref":"oidcc_token.html"},{"type":"module","title":"Records - oidcc_token","doc":"To use the records, import the definition:\n\n```erlang\n-include_lib([\"oidcc/include/oidcc_token.hrl\"]).\n```","ref":"oidcc_token.html#module-records"},{"type":"module","title":"Telemetry - oidcc_token","doc":"See [`Oidcc.Token`](`m:'Elixir.Oidcc.Token'`).","ref":"oidcc_token.html#module-telemetry"},{"type":"function","title":"oidcc_token.authorization_headers/4","doc":"Authorization headers\n\nGenerate a map of authorization headers to use when using the given\naccess token to access an API endpoint.","ref":"oidcc_token.html#authorization_headers/4"},{"type":"function","title":"Examples - oidcc_token.authorization_headers/4","doc":"```erlang\n{ok, ClientContext} =\n    oidcc_client_context:from_configuration_worker(provider_name,\n                                                    <<\"client_id\">>,\n                                                    <<\"client_secret\">>),\n%% Get Access Token record from somewhere\nHeaders =\n    oidcc:authorization_headers(AccessTokenRecord, :get, Url, ClientContext).\n```","ref":"oidcc_token.html#authorization_headers/4-examples"},{"type":"function","title":"oidcc_token.authorization_headers/5","doc":"","ref":"oidcc_token.html#authorization_headers/5"},{"type":"function","title":"oidcc_token.client_credentials/2","doc":"","ref":"oidcc_token.html#client_credentials/2"},{"type":"function","title":"oidcc_token.jwt_profile/4","doc":"Retrieve JSON Web Token (JWT) Profile Token\n\nSee [https://datatracker.ietf.org/doc/html/rfc7523#section-4]\n\nFor a high level interface using {@link oidcc_provider_configuration_worker}\nsee {@link oidcc:jwt_profile_token/6}.","ref":"oidcc_token.html#jwt_profile/4"},{"type":"function","title":"Examples - oidcc_token.jwt_profile/4","doc":"```erlang\n{ok, ClientContext} =\n  oidcc_client_context:from_configuration_worker(provider_name,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>),\n\n{ok, KeyJson} = file:read_file(\"jwt-profile.json\"),\nKeyMap = jose:decode(KeyJson),\nKey = jose_jwk:from_pem(maps:get(<<\"key\">>, KeyMap)),\n\n{ok, #oidcc_token{}} =\n  oidcc_token:jwt_profile(<<\"subject\">>,\n                          ClientContext,\n                          Key,\n                          #{scope => [<<\"scope\">>],\n                            kid => maps:get(<<\"keyId\">>, KeyMap)}).\n```","ref":"oidcc_token.html#jwt_profile/4-examples"},{"type":"function","title":"oidcc_token.refresh/3","doc":"Refresh Token\n\nFor a high level interface using `m:oidcc_provider_configuration_worker`\nsee `oidcc:refresh_token/5`.","ref":"oidcc_token.html#refresh/3"},{"type":"function","title":"Examples - oidcc_token.refresh/3","doc":"```erlang\n{ok, ClientContext} =\n  oidcc_client_context:from_configuration_worker(provider_name,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>),\n\n%% Get AuthCode from Redirect\n\n{ok, Token} =\n  oidcc_token:retrieve(AuthCode, ClientContext, #{\n    redirect_uri => <<\"https://example.com/callback\">>}).\n\n%% Later\n\n{ok, #oidcc_token{}} =\n  oidcc_token:refresh(Token,\n                      ClientContext,\n                      #{expected_subject => <<\"sub_from_initial_id_token\">>}).\n```","ref":"oidcc_token.html#refresh/3-examples"},{"type":"function","title":"oidcc_token.retrieve/3","doc":"Retrieve the token using the authcode received before and directly validate\nthe result.\n\nThe authcode was sent to the local endpoint by the OpenId Connect provider,\nusing redirects.\n\nFor a high level interface using `m:oidcc_provider_configuration_worker`\nsee `oidcc:retrieve_token/5`.","ref":"oidcc_token.html#retrieve/3"},{"type":"function","title":"Examples - oidcc_token.retrieve/3","doc":"```erlang\n{ok, ClientContext} =\n  oidcc_client_context:from_configuration_worker(provider_name,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>),\n\n%% Get AuthCode from Redirect\n\n{ok, #oidcc_token{}} =\n  oidcc:retrieve(AuthCode, ClientContext, #{\n    redirect_uri => <<\"https://example.com/callback\">>}).\n```","ref":"oidcc_token.html#retrieve/3-examples"},{"type":"function","title":"oidcc_token.validate_id_token/3","doc":"","ref":"oidcc_token.html#validate_id_token/3"},{"type":"function","title":"oidcc_token.validate_jarm/3","doc":"Validate the JARM response, returning the valid claims as a map.\n\nThe response was sent to the local endpoint by the OpenId Connect provider,\nusing redirects.","ref":"oidcc_token.html#validate_jarm/3"},{"type":"function","title":"Examples - oidcc_token.validate_jarm/3","doc":"```erlang\n{ok, ClientContext} =\n  oidcc_client_context:from_configuration_worker(provider_name,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>),\n\n%% Get Response from Redirect\n\n{ok, #{<<\"code\">> := AuthCode}} =\n  oidcc:validate_jarm(Response, ClientContext, #{}),\n\n{ok, #oidcc_token{}} = oidcc:retrieve(AuthCode, ClientContext,\n  #{redirect_uri => <<\"https://redirect.example/\">>}).\n```","ref":"oidcc_token.html#validate_jarm/3-examples"},{"type":"function","title":"oidcc_token.validate_jwt/3","doc":"Validate JWT\n\nValidates a generic JWT (such as an access token) from the given provider.\nUseful if the issuer is shared between multiple applications, and the access token\ngenerated for a user at one client is used to validate their access at another client.","ref":"oidcc_token.html#validate_jwt/3"},{"type":"function","title":"Examples - oidcc_token.validate_jwt/3","doc":"```erlang\n{ok, ClientContext} =\n    oidcc_client_context:from_configuration_worker(provider_name,\n                                                <<\"client_id\">>,\n                                                <<\"client_secret\">>),\n%% Get Jwt from Authorization header\n{ok, Claims} =\n    oidcc:validate_jwt(Jwt, ClientContext, Opts).\n```","ref":"oidcc_token.html#validate_jwt/3-examples"},{"type":"type","title":"oidcc_token.access/0","doc":"Access Token Wrapper.","ref":"oidcc_token.html#t:access/0"},{"type":"type","title":"Fields - oidcc_token.access/0","doc":"* `token` - The retrieved token.\n* `expires` - Number of seconds the token is valid.","ref":"oidcc_token.html#t:access/0-fields"},{"type":"type","title":"oidcc_token.authorization_headers_opts/0","doc":"","ref":"oidcc_token.html#t:authorization_headers_opts/0"},{"type":"type","title":"oidcc_token.client_credentials_opts/0","doc":"","ref":"oidcc_token.html#t:client_credentials_opts/0"},{"type":"type","title":"oidcc_token.error/0","doc":"","ref":"oidcc_token.html#t:error/0"},{"type":"type","title":"oidcc_token.id/0","doc":"ID Token Wrapper.","ref":"oidcc_token.html#t:id/0"},{"type":"type","title":"Fields - oidcc_token.id/0","doc":"* `token` - The retrieved token.\n* `claims` - Unpacked claims of the verified token.","ref":"oidcc_token.html#t:id/0-fields"},{"type":"type","title":"oidcc_token.jwt_profile_opts/0","doc":"","ref":"oidcc_token.html#t:jwt_profile_opts/0"},{"type":"type","title":"oidcc_token.refresh/0","doc":"Refresh Token Wrapper.","ref":"oidcc_token.html#t:refresh/0"},{"type":"type","title":"Fields - oidcc_token.refresh/0","doc":"* `token` - The retrieved token.","ref":"oidcc_token.html#t:refresh/0-fields"},{"type":"type","title":"oidcc_token.refresh_opts/0","doc":"","ref":"oidcc_token.html#t:refresh_opts/0"},{"type":"type","title":"oidcc_token.refresh_opts_no_sub/0","doc":"See `t:refresh_opts_no_sub/0`.","ref":"oidcc_token.html#t:refresh_opts_no_sub/0"},{"type":"type","title":"oidcc_token.retrieve_opts/0","doc":"Options for retrieving a token.\n\nSee https://datatracker.ietf.org/doc/html/rfc6749#section-4.1.3.","ref":"oidcc_token.html#t:retrieve_opts/0"},{"type":"type","title":"Fields - oidcc_token.retrieve_opts/0","doc":"* `pkce_verifier` - PKCE verifier (random string previously given to\n  `m:oidcc_authorization`), see\n  https://datatracker.ietf.org/doc/html/rfc7636#section-4.1.\n* `require_pkce` - whether to require PKCE when getting the token.\n* `nonce` - Nonce to check.\n* `scope` - Scope to store with the token.\n* `refresh_jwks` - How to handle tokens with an unknown `kid`.\n  See `t:oidcc_jwt_util:refresh_jwks_for_unknown_kid_fun/0`.\n* `redirect_uri` - Redirect URI given to `oidcc_authorization:create_redirect_url/2`.\n* `dpop_nonce` - if using DPoP, the `nonce` value to use in the proof claim.\n* `trusted_audiences` - if present, a list of additional audience values to\n  accept. Defaults to `any` which allows any additional values.","ref":"oidcc_token.html#t:retrieve_opts/0-fields"},{"type":"type","title":"oidcc_token.t/0","doc":"Token Response Wrapper.","ref":"oidcc_token.html#t:t/0"},{"type":"type","title":"Fields - oidcc_token.t/0","doc":"* `id` - `t:id/0`.\n* `access` - `t:access/0`.\n* `refresh` - `t:refresh/0`.\n* `scope` - `t:oidcc_scope:scopes/0`.","ref":"oidcc_token.html#t:t/0-fields"},{"type":"type","title":"oidcc_token.validate_jarm_opts/0","doc":"Options for refreshing a token.\n\nSee https://datatracker.ietf.org/doc/html/rfc6749#section-4.1.3.","ref":"oidcc_token.html#t:validate_jarm_opts/0"},{"type":"type","title":"Fields - oidcc_token.validate_jarm_opts/0","doc":"* `scope` - Scope to store with the token.\n* `refresh_jwks` - How to handle tokens with an unknown `kid`.\n  See `t:oidcc_jwt_util:refresh_jwks_for_unknown_kid_fun/0`.\n* `expected_subject` - `sub` of the original token.","ref":"oidcc_token.html#t:validate_jarm_opts/0-fields"},{"type":"type","title":"oidcc_token.validate_jwt_opts/0","doc":"","ref":"oidcc_token.html#t:validate_jwt_opts/0"},{"type":"module","title":"oidcc_token_introspection","doc":"OAuth Token Introspection.\n\nSee https://datatracker.ietf.org/doc/html/rfc7662.","ref":"oidcc_token_introspection.html"},{"type":"module","title":"Records - oidcc_token_introspection","doc":"To use the records, import the definition:\n\n```erlang\n-include_lib([\"oidcc/include/oidcc_token_introspection.hrl\"]).\n```","ref":"oidcc_token_introspection.html#module-records"},{"type":"module","title":"Telemetry - oidcc_token_introspection","doc":"See [`Oidcc.TokenIntrospection`](`m:'Elixir.Oidcc.TokenIntrospection'`).","ref":"oidcc_token_introspection.html#module-telemetry"},{"type":"function","title":"oidcc_token_introspection.introspect/3","doc":"Introspect the given access token.\n\nFor a high level interface using `m:oidcc_provider_configuration_worker`\nsee `oidcc:introspect_token/5`.","ref":"oidcc_token_introspection.html#introspect/3"},{"type":"function","title":"Examples - oidcc_token_introspection.introspect/3","doc":"```erlang\n{ok, ClientContext} =\n  oidcc_client_context:from_configuration_worker(provider_name,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>),\n\n%% Get AccessToken\n\n{ok, #oidcc_token_introspection{active = True}} =\n  oidcc_token_introspection:introspect(AccessToken, ClientContext, #{}).\n```","ref":"oidcc_token_introspection.html#introspect/3-examples"},{"type":"type","title":"oidcc_token_introspection.error/0","doc":"","ref":"oidcc_token_introspection.html#t:error/0"},{"type":"type","title":"oidcc_token_introspection.opts/0","doc":"","ref":"oidcc_token_introspection.html#t:opts/0"},{"type":"type","title":"oidcc_token_introspection.t/0","doc":"Introspection Result.\n\nSee https://datatracker.ietf.org/doc/html/rfc7662#section-2.2.","ref":"oidcc_token_introspection.html#t:t/0"},{"type":"module","title":"oidcc_userinfo","doc":"OpenID Connect Userinfo\n\nSee https://openid.net/specs/openid-connect-core-1_0.html#UserInfo","ref":"oidcc_userinfo.html"},{"type":"module","title":"Telemetry - oidcc_userinfo","doc":"See [`Oidcc.Userinfo`](`m:'Elixir.Oidcc.Userinfo'`).","ref":"oidcc_userinfo.html#module-telemetry"},{"type":"function","title":"oidcc_userinfo.retrieve/3","doc":"Load userinfo for the given token\n\nFor a high level interface using `m:oidcc_provider_configuration_worker`, see\n`oidcc:retrieve_userinfo/5`.","ref":"oidcc_userinfo.html#retrieve/3"},{"type":"function","title":"Examples - oidcc_userinfo.retrieve/3","doc":"```erlang\n{ok, ClientContext} =\n  oidcc_client_context:from_configuration_worker(provider_name,\n                                                 <<\"client_id\">>,\n                                                 <<\"client_secret\">>),\n\n%% Get Token\n\n{ok, #{<<\"sub\">> => Sub}} =\n  oidcc_userinfo:retrieve(Token, ClientContext, #{}).\n```","ref":"oidcc_userinfo.html#retrieve/3-examples"},{"type":"type","title":"oidcc_userinfo.error/0","doc":"","ref":"oidcc_userinfo.html#t:error/0"},{"type":"type","title":"oidcc_userinfo.retrieve_opts/0","doc":"Configure userinfo request\n\nSee https://openid.net/specs/openid-connect-core-1_0.html#UserInfoRequest","ref":"oidcc_userinfo.html#t:retrieve_opts/0"},{"type":"type","title":"Parameters - oidcc_userinfo.retrieve_opts/0","doc":"* `refresh_jwks` - How to handle tokens with an unknown `kid`.\n  See `t:oidcc_jwt_util:refresh_jwks_for_unknown_kid_fun/0`\n* `expected_subject` - expected subject for the userinfo\n  (`sub` from id token)\n* `dpop_nonce` - if using DPoP, the `nonce` value to use in the\n    proof claim","ref":"oidcc_userinfo.html#t:retrieve_opts/0-parameters"},{"type":"type","title":"oidcc_userinfo.retrieve_opts_no_sub/0","doc":"See `t:retrieve_opts/0`.","ref":"oidcc_userinfo.html#t:retrieve_opts_no_sub/0"},{"type":"module","title":"Oidcc","doc":"OpenID Connect High Level Interface","ref":"Oidcc.html"},{"type":"module","title":"Setup - Oidcc","doc":"{:ok, _pid} =\n      Oidcc.ProviderConfiguration.Worker.start_link(%{\n      issuer: \"https://accounts.google.com\",\n      name: MyApp.GoogleConfigProvider\n    })\n\nor via a supervisor\n\n    Supervisor.init([\n      {Oidcc.ProviderConfiguration.Worker, %{issuer: \"https://accounts.google.com\"}}\n    ], strategy: :one_for_one)","ref":"Oidcc.html#module-setup"},{"type":"module","title":"Global Configuration - Oidcc","doc":"* `max_clock_skew` (default `0`) - Maximum allowed clock skew for JWT\n  `exp` / `nbf` validation, in seconds","ref":"Oidcc.html#module-global-configuration"},{"type":"function","title":"Oidcc.client_credentials_token/4","doc":"Retrieve Client Credential Token\n\nSee https://datatracker.ietf.org/doc/html/rfc6749#section-1.3.4","ref":"Oidcc.html#client_credentials_token/4"},{"type":"function","title":"Examples - Oidcc.client_credentials_token/4","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://erlef-test-w4a8z2.zitadel.cloud\"\n    ...>   })\n    ...>\n    ...> {:ok, %Oidcc.Token{}} =\n    ...>   Oidcc.client_credentials_token(\n    ...>     pid,\n    ...>     System.fetch_env!(\"CLIENT_CREDENTIALS_CLIENT_ID\"),\n    ...>     System.fetch_env!(\"CLIENT_CREDENTIALS_CLIENT_SECRET\"),\n    ...>     %{scope: [\"openid\"]}\n    ...>   )","ref":"Oidcc.html#client_credentials_token/4-examples"},{"type":"function","title":"Oidcc.create_redirect_url/4","doc":"Create Auth Redirect URL","ref":"Oidcc.html#create_redirect_url/4"},{"type":"function","title":"Examples - Oidcc.create_redirect_url/4","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\"\n    ...> })\n    ...>\n    ...> {:ok, _redirect_uri} =\n    ...>   Oidcc.create_redirect_url(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\",\n    ...>     %{redirect_uri: \"https://my.server/return\"}\n    ...>   )","ref":"Oidcc.html#create_redirect_url/4-examples"},{"type":"function","title":"Oidcc.initiate_logout_url/4","doc":"Create Initiate URI for Relaying Party initiated Logout\n\nSee [https://openid.net/specs/openid-connect-rpinitiated-1_0.html#RPLogout]","ref":"Oidcc.html#initiate_logout_url/4"},{"type":"function","title":"Examples - Oidcc.initiate_logout_url/4","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://erlef-test-w4a8z2.zitadel.cloud\"\n    ...>   })\n    ...>\n    ...> # Get access_token from Oidcc.Token.retrieve/3\n    ...> token = \"token\"\n    ...>\n    ...> {:ok, _redirect_uri} = Oidcc.initiate_logout_url(\n    ...>   token,\n    ...>   pid,\n    ...>   \"client_id\"\n    ...> )","ref":"Oidcc.html#initiate_logout_url/4-examples"},{"type":"function","title":"Oidcc.introspect_token/5","doc":"Introspect the given access token","ref":"Oidcc.html#introspect_token/5"},{"type":"function","title":"Examples - Oidcc.introspect_token/5","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> Oidcc.introspect_token(\n    ...>   \"access_token\",\n    ...>   pid,\n    ...>   \"client_id\",\n    ...>   \"client_secret\"\n    ...> )\n    ...> # => {:ok, %Oidcc.TokenIntrospection{}}","ref":"Oidcc.html#introspect_token/5-examples"},{"type":"function","title":"Oidcc.jwt_profile_token/6","doc":"Retrieve JSON Web Token (JWT) Profile Token\n\nhttps://datatracker.ietf.org/doc/html/rfc7523#section-4","ref":"Oidcc.html#jwt_profile_token/6"},{"type":"function","title":"Examples - Oidcc.jwt_profile_token/6","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://erlef-test-w4a8z2.zitadel.cloud\"\n    ...>   })\n    ...>\n    ...> %{\"key\" => key, \"keyId\" => kid, \"userId\" => subject} = \"JWT_PROFILE\"\n    ...>   |> System.fetch_env!()\n    ...>   |> JOSE.decode()\n    ...>\n    ...> jwk = JOSE.JWK.from_pem(key)\n    ...>\n    ...> {:ok, %Oidcc.Token{}} =\n    ...>   Oidcc.jwt_profile_token(\n    ...>     subject,\n    ...>     pid,\n    ...>     System.fetch_env!(\"CLIENT_ID\"),\n    ...>     \"client_secret\",\n    ...>     jwk,\n    ...>     %{scope: [\"openid\", \"urn:zitadel:iam:org:project:id:zitadel:aud\"], kid: kid}\n    ...>   )","ref":"Oidcc.html#jwt_profile_token/6-examples"},{"type":"function","title":"Oidcc.refresh_token/5","doc":"Refresh Token","ref":"Oidcc.html#refresh_token/5"},{"type":"function","title":"Examples - Oidcc.refresh_token/5","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> # Get refresh_token from redirect\n    ...> refresh_token = \"refresh_token\"\n    ...>\n    ...> Oidcc.refresh_token(\n    ...>   refresh_token,\n    ...>   pid,\n    ...>   \"client_id\",\n    ...>   \"client_secret\",\n    ...>   %{expected_subject: \"sub_from_initial_id_token\"}\n    ...> )\n    ...> # => {:ok, %Oidcc.Token{}}","ref":"Oidcc.html#refresh_token/5-examples"},{"type":"function","title":"Oidcc.retrieve_token/5","doc":"retrieve the token using the authcode received before and directly validate\nthe result.\n\nthe authcode was sent to the local endpoint by the OpenId Connect provider,\nusing redirects","ref":"Oidcc.html#retrieve_token/5"},{"type":"function","title":"Examples - Oidcc.retrieve_token/5","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> # Get auth_code from redirect\n    ...> auth_code = \"auth_code\"\n    ...>\n    ...> Oidcc.retrieve_token(\n    ...>   auth_code,\n    ...>   pid,\n    ...>   \"client_id\",\n    ...>   \"client_secret\",\n    ...>   %{redirect_uri: \"https://my.server/return\"}\n    ...> )\n    ...> # => {:ok, %Oidcc.Token{}}","ref":"Oidcc.html#retrieve_token/5-examples"},{"type":"function","title":"Oidcc.retrieve_userinfo/5","doc":"Load userinfo for the given token","ref":"Oidcc.html#retrieve_userinfo/5"},{"type":"function","title":"Examples - Oidcc.retrieve_userinfo/5","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> # Get access_token from Oidcc.Token.retrieve/3\n    ...> access_token = \"access_token\"\n    ...>\n    ...> Oidcc.retrieve_userinfo(\n    ...>   access_token,\n    ...>   pid,\n    ...>   \"client_id\",\n    ...>   \"client_secret\",\n    ...>   %{expected_subject: \"sub\"}\n    ...> )\n    ...> # => {:ok, %{\"sub\" => \"sub\"}}","ref":"Oidcc.html#retrieve_userinfo/5-examples"},{"type":"module","title":"Oidcc.Authorization","doc":"Functions to start an OpenID Connect Authorization","ref":"Oidcc.Authorization.html"},{"type":"module","title":"Telemetry - Oidcc.Authorization","doc":"* `[:oidcc, :par_request, :start]`\n  * Description: Emitted at the start of executing a PAR request\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :par_request, :stop]`\n  * Description: Emitted at the end of executing a PAR request\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :par_request, :exception]`\n  * Description: Emitted at the end of executing a PAR request\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`","ref":"Oidcc.Authorization.html#module-telemetry"},{"type":"function","title":"Oidcc.Authorization.create_redirect_url/2","doc":"Create Auth Redirect URL\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.create_redirect_url/4`.","ref":"Oidcc.Authorization.html#create_redirect_url/2"},{"type":"function","title":"Examples - Oidcc.Authorization.create_redirect_url/2","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://accounts.google.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> {:ok, _redirect_uri} =\n    ...>   Oidcc.Authorization.create_redirect_url(\n    ...>     client_context,\n    ...>     %{redirect_uri: \"https://my.server/return\"}\n    ...>   )","ref":"Oidcc.Authorization.html#create_redirect_url/2-examples"},{"type":"module","title":"Oidcc.ClientContext","doc":"Client Configuration for authorization, token exchange and userinfo\n\nFor most projects, it makes sense to use\n`Oidcc.ProviderConfiguration.Worker` and the high-level\ninterface of `Oidcc`. In that case direct usage of this\nmodule is not needed.","ref":"Oidcc.ClientContext.html"},{"type":"function","title":"Oidcc.ClientContext.apply_profiles/2","doc":"Apply OpenID Connect / OAuth2 Profiles to the context\n\nSee `:oidcc_client_context.apply_profiles/2` for more.","ref":"Oidcc.ClientContext.html#apply_profiles/2"},{"type":"function","title":"Examples - Oidcc.ClientContext.apply_profiles/2","doc":"iex> {:ok, _pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\",\n    ...>   name: __MODULE__.GoogleConfigProvider\n    ...> })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     __MODULE__.GoogleConfigProvider,\n    ...>     \"client_id\",\n    ...>     \"client_Secret\"\n    ...>   )\n    ...>\n    ...> {:ok, %Oidcc.ClientContext{}, %{}} =\n    ...>   Oidcc.ClientContext.apply_profiles(\n    ...>     client_context,\n    ...>     %{profiles: [:fapi2_message_signing]}\n    ...>   )","ref":"Oidcc.ClientContext.html#apply_profiles/2-examples"},{"type":"function","title":"Oidcc.ClientContext.from_configuration_worker/4","doc":"Create Client Context from a `Oidcc.ProviderConfiguration.Worker`","ref":"Oidcc.ClientContext.html#from_configuration_worker/4"},{"type":"function","title":"Examples - Oidcc.ClientContext.from_configuration_worker/4","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\",\n    ...>   name: __MODULE__.GoogleConfigProvider\n    ...> })\n    ...>\n    ...> {:ok, %Oidcc.ClientContext{}} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     __MODULE__.GoogleConfigProvider,\n    ...>     \"client_id\",\n    ...>     \"client_Secret\"\n    ...>   )\n    ...>\n    ...> {:ok, %Oidcc.ClientContext{}} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_Secret\",\n    ...>     %{client_jwks: JOSE.JWK.generate_key(16)}\n    ...>   )","ref":"Oidcc.ClientContext.html#from_configuration_worker/4-examples"},{"type":"function","title":"Oidcc.ClientContext.from_manual/5","doc":"Create Client Context manually","ref":"Oidcc.ClientContext.html#from_manual/5"},{"type":"function","title":"Examples - Oidcc.ClientContext.from_manual/5","doc":"iex> {:ok, {configuration, _expiry}} =\n    ...>   Oidcc.ProviderConfiguration.load_configuration(\n    ...>     \"https://login.salesforce.com\"\n    ...>   )\n    ...>\n    ...> {:ok, {jwks, _expiry}} =\n    ...>   Oidcc.ProviderConfiguration.load_jwks(\n    ...>     configuration.jwks_uri\n    ...>   )\n    ...>\n    ...> %Oidcc.ClientContext{} =\n    ...>   Oidcc.ClientContext.from_manual(\n    ...>     configuration,\n    ...>     jwks,\n    ...>     \"client_id\",\n    ...>     \"client_Secret\",\n    ...>     %{client_jwks: JOSE.JWK.generate_key(16)}\n    ...>   )","ref":"Oidcc.ClientContext.html#from_manual/5-examples"},{"type":"type","title":"Oidcc.ClientContext.authenticated_t/0","doc":"","ref":"Oidcc.ClientContext.html#t:authenticated_t/0"},{"type":"type","title":"Oidcc.ClientContext.t/0","doc":"","ref":"Oidcc.ClientContext.html#t:t/0"},{"type":"type","title":"Oidcc.ClientContext.unauthenticated_t/0","doc":"","ref":"Oidcc.ClientContext.html#t:unauthenticated_t/0"},{"type":"module","title":"Oidcc.ClientRegistration","doc":"Dynamic Client Registration Utilities","ref":"Oidcc.ClientRegistration.html"},{"type":"module","title":"Telemetry - Oidcc.ClientRegistration","doc":"* `[:oidcc, :register_client, :start]`\n  * Description: Emitted at the start of registering the client\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string()}`\n\n* `[:oidcc, :register_client, :stop]`\n  * Description: Emitted at the end of registering the client\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string()}`\n\n* `[:oidcc, :register_client, :exception]`\n  * Description: Emitted at the end of registering the client\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string()}`","ref":"Oidcc.ClientRegistration.html#module-telemetry"},{"type":"function","title":"Oidcc.ClientRegistration.register/3","doc":"Register Client","ref":"Oidcc.ClientRegistration.html#register/3"},{"type":"function","title":"Examples - Oidcc.ClientRegistration.register/3","doc":"iex> {:ok, {provider_configuration, _expiry}} =\n    ...>   Oidcc.ProviderConfiguration.load_configuration(\"https://accounts.google.com\")\n    ...>\n    ...> Oidcc.ClientRegistration.register(\n    ...>   provider_configuration,\n    ...>   %Oidcc.ClientRegistration{\n    ...>     redirect_uris: [\"https://your.application.com/oidcc/callback\"]\n    ...>   },\n    ...>   %{initial_access_token: \"optional token you got from the provider\"}\n    ...> )\n    ...> # {:ok, %Oidcc.ClientRegistration.Response{\n    ...> #   client_id: client_id,\n    ...> #   client_secret: client_secret\n    ...> # }}","ref":"Oidcc.ClientRegistration.html#register/3-examples"},{"type":"type","title":"Oidcc.ClientRegistration.t/0","doc":"Client Metadata Struct\n\nSee https://openid.net/specs/openid-connect-registration-1_0.html#ClientMetadata and\nhttps://openid.net/specs/openid-connect-rpinitiated-1_0.html#ClientMetadata","ref":"Oidcc.ClientRegistration.html#t:t/0"},{"type":"module","title":"Oidcc.ClientRegistration.Response","doc":"Client Registration Response Struct","ref":"Oidcc.ClientRegistration.Response.html"},{"type":"type","title":"Oidcc.ClientRegistration.Response.t/0","doc":"Client Registration Response Struct\n\nSee https://openid.net/specs/openid-connect-registration-1_0.html#RegistrationResponse","ref":"Oidcc.ClientRegistration.Response.html#t:t/0"},{"type":"module","title":"Oidcc.Logout","doc":"Logout from the OpenID Provider","ref":"Oidcc.Logout.html"},{"type":"function","title":"Oidcc.Logout.initiate_url/3","doc":"Initiate URI for Relaying Party initiated Logout\n\nSee https://openid.net/specs/openid-connect-rpinitiated-1_0.html#RPLogout\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.initiate_logout_url/4`.","ref":"Oidcc.Logout.html#initiate_url/3"},{"type":"function","title":"Examples - Oidcc.Logout.initiate_url/3","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://erlef-test-w4a8z2.zitadel.cloud\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     :unauthenticated\n    ...>   )\n    ...>\n    ...> # Get `token` from `Oidcc.retrieve_token/5`\n    ...> token = \"token\"\n    ...>\n    ...> {:ok, _redirect_uri} =\n    ...>   Oidcc.Logout.initiate_url(\n    ...>     token,\n    ...>     client_context,\n    ...>     %{post_logout_redirect_uri: \"https://my.server/return\"}\n    ...>   )","ref":"Oidcc.Logout.html#initiate_url/3-examples"},{"type":"module","title":"Oidcc.ProviderConfiguration","doc":"Tooling to load and parse Openid Configuration","ref":"Oidcc.ProviderConfiguration.html"},{"type":"module","title":"Telemetry - Oidcc.ProviderConfiguration","doc":"* `[:oidcc, :load_configuration, :start]`\n  * Description: Emitted at the start of loading the provider configuration\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string()}`\n\n* `[:oidcc, :load_configuration, :stop]`\n  * Description: Emitted at the end of loading the provider configuration\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string()}`\n\n* `[:oidcc, :load_configuration, :exception]`\n  * Description: Emitted at the end of loading the provider configuration\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string()}`\n\n* `[:oidcc, :load_jwks, :start]`\n  * Description: Emitted at the start of loading the provider jwks\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{jwks_uri: :uri_string.uri_string()}`\n\n* `[:oidcc, :load_jwks, :stop]`\n  * Description: Emitted at the end of loading the provider jwks\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{jwks_uri: :uri_string.uri_string()}`\n\n* `[:oidcc, :load_jwks, :exception]`\n  * Description: Emitted at the end of loading the provider jwks\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{jwks_uri: :uri_string.uri_string()}`","ref":"Oidcc.ProviderConfiguration.html#module-telemetry"},{"type":"function","title":"Oidcc.ProviderConfiguration.decode_configuration/2","doc":"Decode JSON into OpenID configuration","ref":"Oidcc.ProviderConfiguration.html#decode_configuration/2"},{"type":"function","title":"Examples - Oidcc.ProviderConfiguration.decode_configuration/2","doc":"iex> {:ok, {{~c\"HTTP/1.1\",200, ~c\"OK\"}, _headers, body}} =\n    ...>   :httpc.request(\"https://accounts.google.com/.well-known/openid-configuration\")\n    ...>\n    ...> decoded_json = body |> to_string() |> JOSE.decode()\n    ...>\n    ...> {:ok, %ProviderConfiguration{issuer: \"https://accounts.google.com\"}} =\n    ...>   Oidcc.ProviderConfiguration.decode_configuration(decoded_json)","ref":"Oidcc.ProviderConfiguration.html#decode_configuration/2-examples"},{"type":"function","title":"Oidcc.ProviderConfiguration.load_configuration/2","doc":"Load OpenID Configuration","ref":"Oidcc.ProviderConfiguration.html#load_configuration/2"},{"type":"function","title":"Examples - Oidcc.ProviderConfiguration.load_configuration/2","doc":"iex> {:ok, {\n    ...>   %ProviderConfiguration{issuer: \"https://accounts.google.com\"},\n    ...>   _expiry\n    ...> }} = Oidcc.ProviderConfiguration.load_configuration(\"https://accounts.google.com\")","ref":"Oidcc.ProviderConfiguration.html#load_configuration/2-examples"},{"type":"function","title":"Oidcc.ProviderConfiguration.load_jwks/2","doc":"Load JWKs","ref":"Oidcc.ProviderConfiguration.html#load_jwks/2"},{"type":"function","title":"Examples - Oidcc.ProviderConfiguration.load_jwks/2","doc":"iex> {:ok, {%JOSE.JWK{}, _expiry}} =\n    ...>   Oidcc.ProviderConfiguration.load_jwks(\"https://www.googleapis.com/oauth2/v3/certs\")","ref":"Oidcc.ProviderConfiguration.html#load_jwks/2-examples"},{"type":"type","title":"Oidcc.ProviderConfiguration.t/0","doc":"Configuration Struct\n\nFor details on the fields see:\n* https://openid.net/specs/openid-connect-discovery-1_0.html#ProviderMetadata\n* https://datatracker.ietf.org/doc/html/draft-jones-oauth-discovery-01#section-4.1\n* https://openid.net/specs/openid-connect-rpinitiated-1_0.html#OPMetadata","ref":"Oidcc.ProviderConfiguration.html#t:t/0"},{"type":"module","title":"Oidcc.ProviderConfiguration.Worker","doc":"OIDC Config Provider Worker\n\nLoads and continuously refreshes the OIDC configuration and JWKs","ref":"Oidcc.ProviderConfiguration.Worker.html"},{"type":"module","title":"Usage in Supervisor - Oidcc.ProviderConfiguration.Worker","doc":"```elixir\nSupervisor.init([\n  {Oidcc.ProviderConfiguration.Worker, %{issuer: \"https://accounts.google.com\"}}\n], strategy: :one_for_one)\n```","ref":"Oidcc.ProviderConfiguration.Worker.html#module-usage-in-supervisor"},{"type":"function","title":"Oidcc.ProviderConfiguration.Worker.child_spec/1","doc":"","ref":"Oidcc.ProviderConfiguration.Worker.html#child_spec/1"},{"type":"function","title":"Oidcc.ProviderConfiguration.Worker.get_jwks/1","doc":"Get Parsed Jwks","ref":"Oidcc.ProviderConfiguration.Worker.html#get_jwks/1"},{"type":"function","title":"Examples - Oidcc.ProviderConfiguration.Worker.get_jwks/1","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\"\n    ...> })\n    ...> %JOSE.JWK{} =\n    ...>   Oidcc.ProviderConfiguration.Worker.get_jwks(pid)","ref":"Oidcc.ProviderConfiguration.Worker.html#get_jwks/1-examples"},{"type":"function","title":"Oidcc.ProviderConfiguration.Worker.get_provider_configuration/1","doc":"Get Configuration","ref":"Oidcc.ProviderConfiguration.Worker.html#get_provider_configuration/1"},{"type":"function","title":"Examples - Oidcc.ProviderConfiguration.Worker.get_provider_configuration/1","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\"\n    ...> })\n    ...> %Oidcc.ProviderConfiguration{issuer: \"https://accounts.google.com\"} =\n    ...>   Oidcc.ProviderConfiguration.Worker.get_provider_configuration(pid)","ref":"Oidcc.ProviderConfiguration.Worker.html#get_provider_configuration/1-examples"},{"type":"function","title":"Oidcc.ProviderConfiguration.Worker.refresh_configuration/1","doc":"Refresh Configuration","ref":"Oidcc.ProviderConfiguration.Worker.html#refresh_configuration/1"},{"type":"function","title":"Examples - Oidcc.ProviderConfiguration.Worker.refresh_configuration/1","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\"\n    ...> })\n    ...> :ok = Oidcc.ProviderConfiguration.Worker.refresh_configuration(pid)","ref":"Oidcc.ProviderConfiguration.Worker.html#refresh_configuration/1-examples"},{"type":"function","title":"Oidcc.ProviderConfiguration.Worker.refresh_jwks/1","doc":"Refresh JWKs","ref":"Oidcc.ProviderConfiguration.Worker.html#refresh_jwks/1"},{"type":"function","title":"Examples - Oidcc.ProviderConfiguration.Worker.refresh_jwks/1","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\"\n    ...> })\n    ...> :ok = Oidcc.ProviderConfiguration.Worker.refresh_jwks(pid)","ref":"Oidcc.ProviderConfiguration.Worker.html#refresh_jwks/1-examples"},{"type":"function","title":"Oidcc.ProviderConfiguration.Worker.refresh_jwks_for_unknown_kid/2","doc":"Refresh JWKs if the provided `Kid` is not matching any currently loaded keys","ref":"Oidcc.ProviderConfiguration.Worker.html#refresh_jwks_for_unknown_kid/2"},{"type":"function","title":"Examples - Oidcc.ProviderConfiguration.Worker.refresh_jwks_for_unknown_kid/2","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\"\n    ...> })\n    ...> :ok = Oidcc.ProviderConfiguration.Worker.refresh_jwks_for_unknown_kid(pid, \"kid\")","ref":"Oidcc.ProviderConfiguration.Worker.html#refresh_jwks_for_unknown_kid/2-examples"},{"type":"function","title":"Oidcc.ProviderConfiguration.Worker.start_link/1","doc":"Start Configuration Worker","ref":"Oidcc.ProviderConfiguration.Worker.html#start_link/1"},{"type":"function","title":"Examples - Oidcc.ProviderConfiguration.Worker.start_link/1","doc":"iex> {:ok, _pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>   issuer: \"https://accounts.google.com\",\n    ...>   name: __MODULE__.GoogleConfigProvider\n    ...> })","ref":"Oidcc.ProviderConfiguration.Worker.html#start_link/1-examples"},{"type":"type","title":"Oidcc.ProviderConfiguration.Worker.opts/0","doc":"See `t:oidcc_provider_configuration_worker.opts/0`","ref":"Oidcc.ProviderConfiguration.Worker.html#t:opts/0"},{"type":"module","title":"Oidcc.Token","doc":"Facilitate OpenID Code/Token Exchanges","ref":"Oidcc.Token.html"},{"type":"module","title":"Telemetry - Oidcc.Token","doc":"* `[:oidcc, :request_token, :start]`\n  * Description: Emitted at the start of requesting a code token\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :request_token, :stop]`\n  * Description: Emitted at the end of requesting a code token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :request_token, :exception]`\n  * Description: Emitted at the end of requesting a code token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :refresh_token, :start]`\n  * Description: Emitted at the start of refreshing a token\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :refresh_token, :stop]`\n  * Description: Emitted at the end of refreshing a token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :refresh_token, :exception]`\n  * Description: Emitted at the end of refreshing a token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :jwt_profile_token, :start]`\n  * Description: Emitted at the start of exchanging a JWT profile token\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :jwt_profile_token, :stop]`\n  * Description: Emitted at the end of exchanging a JWT profile token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :jwt_profile_token, :exception]`\n  * Description: Emitted at the end of exchanging a JWT profile token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :client_credentials, :start]`\n  * Description: Emitted at the start of requesting a client credentials token\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :client_credentials, :stop]`\n  * Description: Emitted at the end of requesting a client credentials token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :client_credentials, :exception]`\n  * Description: Emitted at the end of requesting a client credentials token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`","ref":"Oidcc.Token.html#module-telemetry"},{"type":"function","title":"Oidcc.Token.client_credentials/2","doc":"Retrieve Client Credential Token\n\nSee https://datatracker.ietf.org/doc/html/rfc6749#section-1.3.4\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.client_credentials_token/4`.","ref":"Oidcc.Token.html#client_credentials/2"},{"type":"function","title":"Examples - Oidcc.Token.client_credentials/2","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://erlef-test-w4a8z2.zitadel.cloud\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     System.fetch_env!(\"CLIENT_CREDENTIALS_CLIENT_ID\"),\n    ...>     System.fetch_env!(\"CLIENT_CREDENTIALS_CLIENT_SECRET\")\n    ...>   )\n    ...>\n    ...> {:ok, %Oidcc.Token{}} =\n    ...>   Oidcc.Token.client_credentials(\n    ...>     client_context,\n    ...>     %{scope: [\"openid\"]}\n    ...>   )","ref":"Oidcc.Token.html#client_credentials/2-examples"},{"type":"function","title":"Oidcc.Token.jwt_profile/4","doc":"Retrieve JSON Web Token (JWT) Profile Token\n\nSee https://datatracker.ietf.org/doc/html/rfc7523#section-4\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.jwt_profile_token/6`.","ref":"Oidcc.Token.html#jwt_profile/4"},{"type":"function","title":"Examples - Oidcc.Token.jwt_profile/4","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://erlef-test-w4a8z2.zitadel.cloud\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     System.fetch_env!(\"CLIENT_ID\"),\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> %{\"key\" => key, \"keyId\" => kid, \"userId\" => subject} = \"JWT_PROFILE\"\n    ...>   |> System.fetch_env!()\n    ...>   |> JOSE.decode()\n    ...>\n    ...> jwk = JOSE.JWK.from_pem(key)\n    ...>\n    ...> {:ok, %Oidcc.Token{}} =\n    ...>   Oidcc.Token.jwt_profile(\n    ...>     subject,\n    ...>     client_context,\n    ...>     jwk,\n    ...>     %{scope: [\"openid\", \"urn:zitadel:iam:org:project:id:zitadel:aud\"], kid: kid}\n    ...>   )","ref":"Oidcc.Token.html#jwt_profile/4-examples"},{"type":"function","title":"Oidcc.Token.refresh/3","doc":"Refresh Token\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.refresh_token/5`.","ref":"Oidcc.Token.html#refresh/3"},{"type":"function","title":"Examples - Oidcc.Token.refresh/3","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> # Get refresh_token from redirect\n    ...> refresh_token = \"refresh_token\"\n    ...>\n    ...> Oidcc.Token.refresh(\n    ...>   refresh_token,\n    ...>   client_context,\n    ...>   %{expected_subject: \"sub\"}\n    ...> )\n    ...> # => {:ok, %Oidcc.Token{}}","ref":"Oidcc.Token.html#refresh/3-examples"},{"type":"function","title":"Oidcc.Token.retrieve/3","doc":"retrieve the token using the authcode received before and directly validate\nthe result.\n\nthe authcode was sent to the local endpoint by the OpenId Connect provider,\nusing redirects\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.retrieve_token/5`.","ref":"Oidcc.Token.html#retrieve/3"},{"type":"function","title":"Examples - Oidcc.Token.retrieve/3","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> # Get auth_code from redirect\n    ...> auth_code = \"auth_code\"\n    ...>\n    ...> Oidcc.Token.retrieve(\n    ...>   auth_code,\n    ...>   client_context,\n    ...>   %{redirect_uri: \"https://my.server/return\"}\n    ...> )\n    ...> # => {:ok, %Oidcc.Token{}}","ref":"Oidcc.Token.html#retrieve/3-examples"},{"type":"function","title":"Oidcc.Token.validate_id_token/3","doc":"Validate ID Token\n\nUsually the id token is validated using `retrieve/3`.\nIf you get the token passed from somewhere else, this function can validate it.","ref":"Oidcc.Token.html#validate_id_token/3"},{"type":"function","title":"Examples - Oidcc.Token.validate_id_token/3","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> #Get IdToken from somewhere\n    ...> id_token = \"id_token\"\n    ...>\n    ...> Oidcc.Token.validate_id_token(id_token, client_context, :any)\n    ...> # => {:ok, %{\"sub\" => \"sub\", ... }}","ref":"Oidcc.Token.html#validate_id_token/3-examples"},{"type":"function","title":"Oidcc.Token.validate_jarm/3","doc":"Validate the JARM response, returning the valid claims as a map.\n\nthe response was sent to the local endpoint by the OpenId Connect provider,\nusing redirects","ref":"Oidcc.Token.html#validate_jarm/3"},{"type":"function","title":"Examples - Oidcc.Token.validate_jarm/3","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> # Get auth_code from redirect\n    ...> response = \"JWT\"\n    ...>\n    ...> Oidcc.Token.validate_jarm(\n    ...>   response,\n    ...>   client_context,\n    ...>   %{}\n    ...> )\n    ...> # => {:ok, %{\"code\" => auth_code}}","ref":"Oidcc.Token.html#validate_jarm/3-examples"},{"type":"function","title":"Oidcc.Token.validate_jwt/3","doc":"Validate JWT\n\nValidates a generic JWT (such as an access token) from the given provider.\nUseful if the issuer is shared between multiple applications, and the access token\ngenerated for a user at one client is used to validate their access at another client.","ref":"Oidcc.Token.html#validate_jwt/3"},{"type":"function","title":"Examples - Oidcc.Token.validate_jwt/3","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> #Get JWT from Authorization header\n    ...> jwt = \"jwt\"\n    ...>\n    ...> opts = %{\n    ...>   signing_algs: client_context.provider_configuration.id_token_signing_alg_values_supported\n    ...> }\n    ...>\n    ...> Oidcc.Token.validate_jwt(jwt, client_context, opts)\n    ...> # => {:ok, %{\"sub\" => \"sub\", ... }}","ref":"Oidcc.Token.html#validate_jwt/3-examples"},{"type":"type","title":"Oidcc.Token.t/0","doc":"","ref":"Oidcc.Token.html#t:t/0"},{"type":"module","title":"Oidcc.Token.Access","doc":"Access Token struct.\n\nSee `t::oidcc_token.access/0`","ref":"Oidcc.Token.Access.html"},{"type":"function","title":"Oidcc.Token.Access.authorization_headers/5","doc":"Generate a map of authorization headers to use when using the given\n`Oidcc.Token.Access` struct to access an API endpoint.","ref":"Oidcc.Token.Access.html#authorization_headers/5"},{"type":"type","title":"Oidcc.Token.Access.t/0","doc":"","ref":"Oidcc.Token.Access.html#t:t/0"},{"type":"module","title":"Oidcc.Token.Id","doc":"ID Token struct","ref":"Oidcc.Token.Id.html"},{"type":"type","title":"Oidcc.Token.Id.t/0","doc":"","ref":"Oidcc.Token.Id.html#t:t/0"},{"type":"module","title":"Oidcc.Token.Refresh","doc":"Refresh Token struct","ref":"Oidcc.Token.Refresh.html"},{"type":"type","title":"Oidcc.Token.Refresh.t/0","doc":"","ref":"Oidcc.Token.Refresh.html#t:t/0"},{"type":"module","title":"Oidcc.TokenIntrospection","doc":"OAuth Token Introspection\n\nSee https://datatracker.ietf.org/doc/html/rfc7662","ref":"Oidcc.TokenIntrospection.html"},{"type":"module","title":"Telemetry - Oidcc.TokenIntrospection","doc":"* `[:oidcc, :introspect_token, :start]`\n  * Description: Emitted at the start of introspecting the token\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :introspect_token, :stop]`\n  * Description: Emitted at the end of introspecting the token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :introspect_token, :exception]`\n  * Description: Emitted at the end of introspecting the token\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`","ref":"Oidcc.TokenIntrospection.html#module-telemetry"},{"type":"function","title":"Oidcc.TokenIntrospection.introspect/3","doc":"Introspect the given access token\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.introspect_token/5`.","ref":"Oidcc.TokenIntrospection.html#introspect/3"},{"type":"function","title":"Examples - Oidcc.TokenIntrospection.introspect/3","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> Oidcc.TokenIntrospection.introspect(\n    ...>   \"access_token\",\n    ...>   client_context\n    ...> )\n    ...> # => {:ok, %Oidcc.TokenIntrospection{}}","ref":"Oidcc.TokenIntrospection.html#introspect/3-examples"},{"type":"type","title":"Oidcc.TokenIntrospection.t/0","doc":"For details on the fields see:\n* https://datatracker.ietf.org/doc/html/rfc7662#section-2.2","ref":"Oidcc.TokenIntrospection.html#t:t/0"},{"type":"module","title":"Oidcc.Userinfo","doc":"OpenID Connect Userinfo\n\nSee https://openid.net/specs/openid-connect-core-1_0.html#UserInfo","ref":"Oidcc.Userinfo.html"},{"type":"module","title":"Telemetry - Oidcc.Userinfo","doc":"* `[:oidcc, :userinfo, :start]`\n  * Description: Emitted at the start of loading userinfo\n  * Measurements: `%{system_time: non_neg_integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :userinfo, :stop]`\n  * Description: Emitted at the end of loading userinfo\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`\n\n* `[:oidcc, :userinfo, :exception]`\n  * Description: Emitted at the end of loading userinfo\n  * Measurements: `%{duration: integer(), monotonic_time: integer()}`\n  * Metadata: `%{issuer: :uri_string.uri_string(), client_id: String.t()}`","ref":"Oidcc.Userinfo.html#module-telemetry"},{"type":"function","title":"Oidcc.Userinfo.retrieve/3","doc":"Load userinfo for the given token\n\nFor a high level interface using `Oidcc.ProviderConfiguration.Worker`\nsee `Oidcc.retrieve_userinfo/5`.","ref":"Oidcc.Userinfo.html#retrieve/3"},{"type":"function","title":"Examples - Oidcc.Userinfo.retrieve/3","doc":"iex> {:ok, pid} =\n    ...>   Oidcc.ProviderConfiguration.Worker.start_link(%{\n    ...>     issuer: \"https://api.login.yahoo.com\"\n    ...>   })\n    ...>\n    ...> {:ok, client_context} =\n    ...>   Oidcc.ClientContext.from_configuration_worker(\n    ...>     pid,\n    ...>     \"client_id\",\n    ...>     \"client_secret\"\n    ...>   )\n    ...>\n    ...> # Get access_token from Oidcc.Token.retrieve/3\n    ...> access_token = \"access_token\"\n    ...>\n    ...> Oidcc.Userinfo.retrieve(\n    ...>   access_token,\n    ...>   client_context,\n    ...>   %{expected_subject: \"sub\"}\n    ...> )\n    ...> # => {:ok, %{\"sub\" => \"sub\"}}","ref":"Oidcc.Userinfo.html#retrieve/3-examples"},{"type":"extras","title":"README","doc":"# oidcc\n\nOpenID Connect client library for Erlang.\n\n[![EEF Security WG project](https://img.shields.io/badge/EEF-Security-black)](https://github.com/erlef/security-wg)\n[![Main Branch](https://github.com/erlef/oidcc/actions/workflows/branch_main.yml/badge.svg?branch=main)](https://github.com/erlef/oidcc/actions/workflows/branch_main.yml)\n[![Module Version](https://img.shields.io/hexpm/v/oidcc.svg)](https://hex.pm/packages/oidcc)\n[![Total Download](https://img.shields.io/hexpm/dt/oidcc.svg)](https://hex.pm/packages/oidcc)\n[![License](https://img.shields.io/hexpm/l/oidcc.svg)](https://github.com/erlef/oidcc/blob/main/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/erlef/oidcc.svg)](https://github.com/erlef/oidcc/commits/master)\n[![Coverage Status](https://coveralls.io/repos/github/erlef/oidcc/badge.svg?branch=main)](https://coveralls.io/github/erlef/oidcc?branch=main)\n\n \n\n \n   \n   \n   \n \n\nOpenID Certified by [Jonatan Männchen](https://github.com/maennchen) at the\n[Erlang Ecosystem Foundation](https://github.com/erlef) of multiple Relaying\nParty conformance profiles of the OpenID Connect protocol:\nFor details, check the\n[Conformance Test Suite](https://github.com/erlef/oidcc_conformance).\n\n \n\n \n   \n   \n   \n \n\nThe refactoring for `v3` and the certification is funded as an\n[Erlang Ecosystem Foundation](https://erlef.org/) stipend entered by the\n[Security Working Group](https://erlef.org/wg/security).\n\n \n\n\n \n   \n   \n   \n \n\nA security audit was performed by [SAFE-Erlang-Elixir](https://github.com/SAFE-Erlang-Elixir) more info [HERE](https://www.erlang-solutions.com/landings/security-audit-for-erlang-2/).","ref":"readme.html"},{"type":"extras","title":"Supported Features - README","doc":"* [Discovery](https://openid.net/specs/openid-connect-discovery-1_0.html)\n  (`[ISSUER]/.well-known/openid-configuration`)\n* [Client Registration](https://openid.net/specs/openid-connect-registration-1_0.html)\n* Authorization (Code Flow)\n  * [Request Object](https://openid.net/specs/openid-connect-core-1_0.html#RequestObject)\n  * [PKCE](https://oauth.net/2/pkce/)\n  * [Pushed Authorization Requests](https://datatracker.ietf.org/doc/html/rfc9126)\n  * [Authorization Server Issuer Identification](https://datatracker.ietf.org/doc/html/rfc9207)\n* Token\n  * Authorization: `client_secret_basic`, `client_secret_post`,\n    `client_secret_jwt`, and `private_key_jwt`\n  * Grant Types: `authorization_code`, `refresh_token`, `jwt_bearer`, and\n    `client_credentials`\n  * Automatic JWK Refreshing when needed\n* Userinfo\n  * [JWT Response](https://openid.net/specs/openid-connect-core-1_0.html#UserInfoResponse)\n  * [Aggregated and Distributed Claims](https://openid.net/specs/openid-connect-core-1_0.html#AggregatedDistributedClaims)\n* [Token Introspection](https://datatracker.ietf.org/doc/html/rfc7662)\n* Logout\n  * [RP-Initiated](https://openid.net/specs/openid-connect-rpinitiated-1_0.html)\n* [JWT Secured Authorization Response Mode for OAuth 2.0 (JARM)](https://openid.net/specs/oauth-v2-jarm-final.html)\n* [Demonstrating Proof of Possession (DPoP)](https://datatracker.ietf.org/doc/html/rfc9449)\n* [OAuth 2 Purpose Request Parameter](https://cdn.connectid.com.au/specifications/oauth2-purpose-01.html)\n* Profiles\n  * [FAPI 2.0 Security Profile](https://openid.bitbucket.io/fapi/fapi-2_0-security-profile.html)\n  * [FAPI 2.0 Message Signing](https://openid.bitbucket.io/fapi/fapi-2_0-message-signing.html)","ref":"readme.html#supported-features"},{"type":"extras","title":"Setup - README","doc":"**Please note that the minimum supported Erlang OTP version is OTP26.**","ref":"readme.html#setup"},{"type":"extras","title":"Erlang - README","doc":"**directly**\n\n```erlang\n{ok, Pid} =\n    oidcc_provider_configuration_worker:start_link(#{\n        issuer => <<\"https://accounts.google.com\">>,\n        name => {local, google_config_provider}\n    }).\n```\n\n**via `supervisor`**\n\n```erlang\n-behaviour(supervisor).\n\n%% ...\n\ninit(_Args) ->\n    SupFlags = #{strategy => one_for_one},\n    ChildSpecs = [\n        #{\n            id => oidcc_provider_configuration_worker,\n            start =>\n                {oidcc_provider_configuration_worker, start_link, [\n                    #{\n                        issuer => \"https://accounts.google.com\",\n                        name => {local, myapp_oidcc_config_provider}\n                    }\n                ]},\n            shutdown => brutal_kill\n        }\n    ],\n    {ok, {SupFlags, ChildSpecs}}.\n```","ref":"readme.html#erlang"},{"type":"extras","title":"Elixir - README","doc":"**directly**\n\n```elixir\n{:ok, _pid} =\n  Oidcc.ProviderConfiguration.Worker.start_link(%{\n    issuer: \"https://accounts.google.com\",\n    name: Myapp.OidccConfigProvider\n  })\n```\n\n**via `Supervisor`**\n\n```elixir\nSupervisor.init(\n  [\n    {Oidcc.ProviderConfiguration.Worker,\n     %{\n       issuer: \"https://accounts.google.com\",\n       name: Myapp.OidccConfigProvider\n     }}\n  ],\n  strategy: :one_for_one\n)\n```","ref":"readme.html#elixir"},{"type":"extras","title":"Usage - README","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Companion libraries - README","doc":"`oidcc` offers integrations for various libraries:\n\n<!-- TODO: Uncomment when available -->\n\n- [`oidcc_cowboy`](https://hex.pm/packages/oidcc_cowboy) - Integrations for\n  [`cowboy`](https://hex.pm/packages/cowboy)\n- [`oidcc_plug`](https://hex.pm/packages/oidcc_plug) - Integrations for\n  [`plug`](https://hex.pm/packages/plug) and\n  [`phoenix`](https://hex.pm/packages/phoenix)\n- [`phx_gen_oidcc`](https://hex.pm/packages/phx_gen_oidcc) - Setup Generator for\n  [`phoenix`](https://hex.pm/packages/phoenix)\n- [`ueberauth_oidcc`](https://hex.pm/packages/ueberauth_oidcc) - Integration for\n  [`ueberauth`](https://hex.pm/packages/ueberauth)","ref":"readme.html#companion-libraries"},{"type":"extras","title":"Erlang - README","doc":"```erlang\n%% Create redirect URI for authorization\n{ok, RedirectUri} = oidcc:create_redirect_url(\n    myapp_oidcc_config_provider,\n    <<\"client_id\">>,\n    <<\"client_secret\">>,\n    #{redirect_uri => <<\"https://example.com/callback\">>}\n),\n\n%% Redirect user to `RedirectUri`\n\n%% Retrieve `code` query / form param from redirect back\n\n%% Exchange code for token\n{ok, Token} =\n    oidcc:retrieve_token(\n        AuthCode,\n        myapp_oidcc_config_provider,\n        <<\"client_id\">>,\n        <<\"client_secret\">>,\n        #{redirect_uri => <<\"https://example.com/callback\">>}\n    ),\n\n%% Load userinfo for token\n{ok, Claims} =\n    oidcc:retrieve_userinfo(\n        Token,\n        myapp_oidcc_config_provider,\n        <<\"client_id\">>,\n        <<\"client_secret\">>,\n        #{}\n    ),\n\n%% Load introspection for access token\n{ok, Introspection} =\n    oidcc:introspect_token(\n        Token,\n        myapp_oidcc_config_provider,\n        <<\"client_id\">>,\n        <<\"client_secret\">>,\n        #{}\n    ),\n\n%% Refresh token when it expires\n{ok, RefreshedToken} =\n    oidcc:refresh_token(\n        Token,\n        myapp_oidcc_config_provider,\n        <<\"client_id\">>,\n        <<\"client_secret\">>,\n        #{}\n    ).\n```\n\nfor more details, see https://hexdocs.pm/oidcc/oidcc.html","ref":"readme.html#erlang"},{"type":"extras","title":"Elixir - README","doc":"```elixir\n# Create redirect URI for authorization\n{:ok, redirect_uri} =\n  Oidcc.create_redirect_url(\n    Myapp.OidccConfigProvider,\n    \"client_id\",\n    \"client_secret\",\n    %{redirect_uri: \"https://example.com/callback\"}\n  )\n\n# Redirect user to `redirect_uri`\n\n# Retrieve `code` query / form param from redirect back\n\n# Exchange code for token\n{:ok, token} =\n  Oidcc.retrieve_token(\n    auth_code,\n    Myapp.OidccConfigProvider,\n    \"client_id\",\n    \"client_secret\",\n    %{redirect_uri: \"https://example.com/callback\"}\n  )\n\n# Load userinfo for token\n{:ok, claims} =\n  Oidcc.retrieve_userinfo(\n    token,\n    Myapp.OidccConfigProvider,\n    \"client_id\",\n    \"client_secret\",\n    %{expected_subject: \"sub\"}\n  )\n\n# Load introspection for access token\n{:ok, introspection} =\n  Oidcc.introspect_token(\n    token,\n    Myapp.OidccConfigProvider,\n    \"client_id\",\n    \"client_secret\"\n  )\n\n# Refresh token when it expires\n{:ok, refreshed_token} =\n  Oidcc.refresh_token(\n    token,\n    Myapp.OidccConfigProvider,\n    \"client_id\",\n    \"client_secret\"\n  )\n```\n\nfor more details, see https://hexdocs.pm/oidcc/Oidcc.html","ref":"readme.html#elixir"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}